@@ -1,6 +1,6 @@
 package POE::Component::Client::HTTP;
-BEGIN {
-  $POE::Component::Client::HTTP::VERSION = '0.943';
+{
+  $POE::Component::Client::HTTP::VERSION = '0.948';
 }
 # vim: ts=2 sw=2 expandtab
 
@@ -15,7 +15,10 @@ use constant DEBUG_DATA => 0;
 use Carp qw(croak);
 use HTTP::Response;
 use Net::HTTP::Methods;
-use Socket qw(sockaddr_in inet_ntoa);
+use Socket qw(
+  sockaddr_in inet_ntoa
+  getnameinfo NI_NUMERICHOST NI_NUMERICSERV
+);
 
 use POE::Component::Client::HTTP::RequestFactory;
 use POE::Component::Client::HTTP::Request qw(:states :fields);
@@ -30,8 +33,6 @@ BEGIN {
   };
 }
 
-use Socket::GetAddrInfo qw(:newapi getnameinfo NI_NUMERICHOST NI_NUMERICSERV);
-
 use POE qw(
   Driver::SysRW Filter::Stream
   Filter::HTTPHead Filter::HTTPChunk
@@ -210,12 +211,10 @@ sub _poco_weeble_request {
     $proxy_override
   ) = @_[KERNEL, HEAP, SENDER, ARG0, ARG1, ARG2, ARG3, ARG4];
 
+  my $scheme = $http_request->uri->scheme;
   unless (
-    defined($http_request->uri->scheme) and
-    length($http_request->uri->scheme) and
-    $supported_schemes{$http_request->uri->scheme} and
-    defined($http_request->uri->host) and
-    length($http_request->uri->host)
+    defined($scheme) and
+    exists $supported_schemes{$scheme}
   ) {
     my $rsp = HTTP::Response->new(
        400 => 'Bad Request', [],
@@ -223,12 +222,33 @@ sub _poco_weeble_request {
        . "<HEAD><TITLE>Error: Bad Request</TITLE></HEAD>\n"
        . "<BODY>\n"
        . "<H1>Error: Bad Request</H1>\n"
-       . "Unsupported URI scheme\n"
+       . "Unsupported URI scheme: '$scheme'\n"
        . "</BODY>\n"
        . "</HTML>\n"
     );
     $rsp->request($http_request);
-    if (ref $response_event) {
+    if (ref($response_event) eq 'POE::Component::Client::HTTP::Request') {
+      $response_event->postback->($rsp);
+    } else {
+      $kernel->post($sender, $response_event, [$http_request, $tag], [$rsp]);
+    }
+    return;
+  }
+
+  my $host = $http_request->uri->host;
+  unless (defined $host and length $host) {
+    my $rsp = HTTP::Response->new(
+       400 => 'Bad Request', [],
+       "<html>\n"
+       . "<HEAD><TITLE>Error: Bad Request</TITLE></HEAD>\n"
+       . "<BODY>\n"
+       . "<H1>Error: Bad Request</H1>\n"
+       . "URI contains no discernable host.\n"
+       . "</BODY>\n"
+       . "</HTML>\n"
+    );
+    $rsp->request($http_request);
+    if (ref($response_event) eq 'POE::Component::Client::HTTP::Request') {
       $response_event->postback->($rsp);
     } else {
       $kernel->post($sender, $response_event, [$http_request, $tag], [$rsp]);
@@ -249,7 +269,7 @@ sub _poco_weeble_request {
        . "</HTML>\n"
       );
     $rsp->request($http_request);
-    if (ref $response_event) {
+    if (ref($response_event) eq 'POE::Component::Client::HTTP::Request') {
       $response_event->postback->($rsp);
     } else {
       $kernel->post($sender, $response_event, [$http_request, $tag], [$rsp]);
@@ -631,14 +651,17 @@ sub _poco_weeble_io_read {
   DEBUG_DATA and warn (ref($input) ? $input->as_string : _hexdump($input));
 
   # There was no corresponding request?  Nothing left to do here.
-  # We might have got here because the server sent EOF after we were done processing
-  # the request, and deleted it from our cache. ( notes for RT#50231 )
+  #
+  # We might have got here because the server sent EOF after we were
+  # done processing the request, and deleted it from our cache. (
+  # notes for RT#50231 )
   return unless defined $request_id;
 
   my $request = $heap->{request}->{$request_id};
   return unless defined $request;
   DEBUG and warn(
-    "REQUEST $request_id is $request <" . $request->[REQ_HTTP_REQUEST]->uri . ">"
+    "REQUEST $request_id is $request <",
+    $request->[REQ_HTTP_REQUEST]->uri(), ">"
   );
 
   # Reset the timeout if we get data.
@@ -828,7 +851,7 @@ sub _poco_weeble_io_read {
       # $request->close_connection;
     }
     else {
-      my $is_done = $request->add_content ($input);
+      $request->add_content($input);
     }
   }
 
@@ -1051,7 +1074,7 @@ POE::Component::Client::HTTP - a HTTP user-agent component
 
 =head1 VERSION
 
-version 0.943
+version 0.948
 
 =head1 SYNOPSIS
 
@@ -1344,12 +1367,12 @@ Requests are posted to the component's "request" state.  They include
 an HTTP::Request object which defines the request.  For example:
 
   $kernel->post(
-    'ua', 'request',           # http session alias & state
-    'response',                # my state to receive responses
-    GET 'http://poe.perl.org', # a simple HTTP request
-    'unique id',               # a tag to identify the request
-    'progress',                # an event to indicate progress
-    'http://1.2.3.4:80/'       # proxy to use for this request
+    'ua', 'request',            # http session alias & state
+    'response',                 # my state to receive responses
+    GET('http://poe.perl.org'), # a simple HTTP request
+    'unique id',                # a tag to identify the request
+    'progress',                 # an event to indicate progress
+    'http://1.2.3.4:80/'        # proxy to use for this request
   );
 
 Requests include the state to which responses will be posted.  In the
@@ -1604,13 +1627,14 @@ https://rt.cpan.org/Dist/Display.html?Queue=POE-Component-Client-HTTP
 
 =head1 REPOSITORY
 
-http://github.com/rcaputo/poe-component-client-http
-http://gitorious.org/poe-component-client-http
+Github: L<http://github.com/rcaputo/poe-component-client-http> .
+
+Gitorious: L<http://gitorious.org/poe-component-client-http> .
 
 =head1 OTHER RESOURCES
 
-http://search.cpan.org/dist/POE-Component-Client-HTTP/
+L<http://search.cpan.org/dist/POE-Component-Client-HTTP/>
 
 =cut
 
-# }}} POD
\ No newline at end of file
+# }}} POD
