@@ -1,4 +1,7 @@
 package POE::Component::Client::HTTP::Request;
+{
+  $POE::Component::Client::HTTP::Request::VERSION = '0.948';
+}
 # vim: ts=2 sw=2 expandtab
 
 use strict;
@@ -8,6 +11,7 @@ use POE;
 
 use Carp;
 use HTTP::Status;
+use Errno qw(ETIMEDOUT);
 
 BEGIN {
   local $SIG{'__DIE__'} = 'DEFAULT';
@@ -24,6 +28,10 @@ my $request_seq = 0;
 
 use constant DEBUG => 0;
 
+# TODO CONNECT - Add a flag to indicate whether to generate an HTTP
+# CONNECT request for proxying, or to return REQ_HTTP_REQUEST.  Add a
+# method to update that flag.
+
 use constant REQ_ID            =>  0;
 use constant REQ_POSTBACK      =>  1;
 use constant REQ_CONNECTION    =>  2;
@@ -120,11 +128,13 @@ sub new {
 
   # Add a host header if one isn't included.  Must do this before
   # we reset the $host for the proxy!
-  _set_host_header ($http_request) unless (
+  unless (
     defined $http_request->header('Host') and
     length $http_request->header('Host')
-  );
-
+  ) {
+    my $error = _set_host_header($http_request);
+    croak "Can't set Host header: $error" if $error;
+  }
 
   if (defined $params{Proxy}) {
     # This request qualifies for proxying.  Replace the host and port
@@ -282,7 +292,6 @@ sub add_content {
 
     $self->[REQ_STATE] |= RS_DONE;
     $self->[REQ_STATE] &= ~RS_IN_CONTENT;
-    $self->[REQ_CONNECTION]->close();
     return 1;
   }
 
@@ -398,7 +407,9 @@ sub _set_host_header {
       $request->header( Host => "$new_host:$new_port" );
     }
   };
-  warn $@ if $@;
+
+  # Return Boolean state of the eval.
+  return $@;
 }
 
 sub does_redirect {
@@ -429,11 +440,12 @@ sub check_redirect {
   # Make sure to frob any cookies set.  Redirect cookies are cookies, too!
   $self->[REQ_FACTORY]->frob_cookies($self->[REQ_RESPONSE]);
 
-  my $new_uri = $self->[REQ_RESPONSE]->header ('Location');
-  DEBUG and warn "REQ: Preparing redirect to $new_uri";
+  my $location_uri = $self->[REQ_RESPONSE]->header('Location');
+
+  DEBUG and warn "REQ: Preparing redirect to $location_uri";
   my $base = $self->[REQ_RESPONSE]->base();
-  $new_uri = URI->new($new_uri, $base)->abs($base);
-  DEBUG and warn "RED: Actual redirect uri is $new_uri";
+  $location_uri = URI->new($location_uri, $base)->abs($base);
+  DEBUG and warn "RED: Actual redirect uri is $location_uri";
 
   my $prev = $self;
   my $history = 0;
@@ -453,8 +465,18 @@ sub check_redirect {
     $newrequest->remove_header('Cookie');
 
     DEBUG and warn "RED: new request $newrequest";
-    $newrequest->uri($new_uri);
-    _set_host_header ($newrequest);
+    $newrequest->uri($location_uri);
+
+    # Don't change the Host header on a relative redirect.  This
+    # allows the HTTP::Request's Host to remain intact, per
+    # rt.cpan.org #63990.
+    if (defined $location_uri->scheme()) {
+      DEBUG and warn "RED: redirecting to absolute location $location_uri";
+      _set_host_header($newrequest);
+    }
+    else {
+      DEBUG and warn "RED: no new Host for relative redirect to $location_uri";
+    }
 
     $self->[REQ_STATE] = RS_REDIRECTED;
     DEBUG and warn "RED: new request $newrequest";
@@ -526,8 +548,8 @@ sub error {
 
   my $nl = "\n";
 
-  my $r = HTTP::Response->new($code);
-  my $http_msg = status_message ($code);
+  my $http_msg = status_message($code);
+  my $r = HTTP::Response->new($code, $http_msg, [ 'X-PCCH-Errmsg', $message ]);
   my $m = (
     "<html>$nl"
     . "<HEAD><TITLE>Error: $http_msg</TITLE></HEAD>$nl"
@@ -539,20 +561,28 @@ sub error {
     . "</HTML>$nl"
   );
 
-  $r->content ($m);
-  $r->request ($self->[REQ_HTTP_REQUEST]);
+  $r->content($m);
+  $r->request($self->[REQ_HTTP_REQUEST]);
   $self->[REQ_POSTBACK]->($r);
   $self->[REQ_STATE] |= RS_POSTED;
 }
 
 sub connect_error {
-  my ($self, $message) = @_;
+  my ($self, $operation, $errnum, $errstr) = @_;
 
   my $host = $self->[REQ_HOST];
   my $port = $self->[REQ_PORT];
 
-  $message = "Cannot connect to $host:$port ($message)";
-  $self->error (RC_INTERNAL_SERVER_ERROR, $message);
+  if ($operation eq "connect" and $errnum == ETIMEDOUT) {
+    $self->error(408, "Connection to $host:$port failed: timeout");
+  }
+  else {
+    $self->error(
+      RC_INTERNAL_SERVER_ERROR,
+      "Connection to $host:$port failed: $operation error $errnum: $errstr"
+    );
+  }
+
   return;
 }
 
@@ -581,6 +611,10 @@ __END__
 
 POE::Component::Client::HTTP::Request - an HTTP request class
 
+=head1 VERSION
+
+version 0.948
+
 =head1 SYNOPSIS
 
   # Used internally by POE::Component::Client::HTTP
